<?php 
namespace App\Helpers;

use Illuminate\Support\Facades\DB;
use Http;

class MCFHelper{
    
    protected static $url='https://sellingpartnerapi-na.amazon.com/fba/outbound/2020-07-01/fulfillmentOrders/';
    protected static $feedurl='https://sellingpartnerapi-na.amazon.com/feeds/2021-06-30';
    
    public static function getAccessToken($clientId,$clientSecret,$refreshToken,$jsonFile){
        $response = Http::asForm()->post('https://api.amazon.com/auth/o2/token', [
            'client_id' =>$clientId,
            'client_secret' => $clientSecret,
            'grant_type' => 'refresh_token',
            'refresh_token' => $refreshToken,
        ]);

        if ($response->successful()) {
            $tokenData = [
                'access_token' => $response['access_token'],
            ];
            file_put_contents($jsonFile, json_encode($tokenData, JSON_PRETTY_PRINT));

            return response()->json([
                'status' => true,
                'msg' => 'success',
            ]);
        } else {
            return response()->json([
                'status' => false,
                'data' => $response->json(),
            ]);
        }
    }
    
    public static function fullFillmentPreview($access_token,$marketplace_id,$input_data){
        $api_endpoint=self::$url.'preview';
        $userAgent = \Request::server('HTTP_USER_AGENT');
        
        
        // Define your request data
        $requestData = [
            'marketplaceId' => $marketplace_id,
            'address' => [
                'name' => $input_data['address_name'],
                'addressLine1' => $input_data['address_line1'],
                'addressLine2' => $input_data['address_line2'],
                'city' => $input_data['city'],
                'stateOrRegion' => $input_data['state_code'],
                'postalCode' => $input_data['zip_code'],
                'countryCode' => $input_data['country_code'],
            ],
            'shippingSpeedCategories' => [
                'Standard',
                'Expedited',
            ],
            'items' => [
                [
                    'quantity' => $input_data['quantity'],
                    'sellerFulfillmentOrderItemId' => $input_data['order_no'],
                    'sellerSku' => $input_data['amazon_sku'],
                ],
            ],
        ];
        
        // dd($requestData);
        
        // Convert the request data to JSON
        $jsonData = json_encode($requestData);
        
        // Make the POST request to the SP API endpoint
        $response =  Http::withHeaders([
            'x-amz-access-token' => $access_token,
            'Content-Type' => 'application/json',
            'User-Agent' => $userAgent,
        ])->post($api_endpoint, $jsonData);
        
        $responseData = $response->json();
        $filePath=storage_path('amazonspapi/mcf/fullfillment_preview.json');
        file_put_contents($filePath, json_encode($responseData, JSON_PRETTY_PRINT));
        
        // echo json_encode($responseData, JSON_PRETTY_PRINT);exit;
        
         $arr=[];
        if ($response->status() === 200) {
            // Request was successful
               
                $count=1;
                foreach ($responseData['payload']['fulfillmentPreviews'] as $fulfillmentPreview) {
                    $feesSum = 0;
                    $earliestArrivalDate='';
                    $message='No Fullfillable Order May be Inventory Unavailable or any Other Reason';
                    if($fulfillmentPreview['isFulfillable'] ==true){
                        $estimatedFees = $fulfillmentPreview['estimatedFees'];
                        // Iterate through estimatedFees and sum the 'amount.value' values
                        foreach ($estimatedFees as $fee) {
                            $feeValue = (float) $fee['amount']['value'];
                            $feesSum += round($feeValue,2);
                        }
                        $earliestArrivalDate = $fulfillmentPreview['fulfillmentPreviewShipments'][0]['earliestArrivalDate'];
                        $message='Fullfillable Order Possible';
                    }


                    
                    $arr['shiping_speed'.$count]=$fulfillmentPreview['shippingSpeedCategory'];
                    $arr['isFulfillable'.$count] = $fulfillmentPreview['isFulfillable'];
                    $arr['estimatedtotal'.$count] = number_format($feesSum,2);
                    $arr['shippingdate'.$count] = $earliestArrivalDate;
                    $arr['isFulfillablemsg'.$count] = $message;
                    
                    $count++;
                }
                
                $data=[
                    'error' => false,
                    'data' => $arr,
                    'message' => 'Successfully working'
                ];
        } else {
            $error=$responseData['errors'][0]['message'];
            $data=[
                'error' => true,
                'data' => $arr,
                'message' => $error
            ];
        }
        
        return $data;
        
    } 
    
    public static function createFullFillment($access_token,$marketplace_id,$input_data){
        $api_endpoint=self::$url;
        // dd($access_token);
        
        $userAgent = \Request::server('HTTP_USER_AGENT');
        
        // Define your request data
        $requestData = [
            'marketplaceId' => $marketplace_id,
            'displayableOrderId' => $input_data['order_no'],
            'displayableOrderDate' => gmdate("Y-m-d\TH:i:s", time()),
            'displayableOrderComment' => 'Do not leave the package at the door step',
            'sellerFulfillmentOrderId' => $input_data['order_no'],
            'notificationEmails' => ['moses.esire@gmail.com'],
            'destinationAddress' => [
                'name' => $input_data['address_name'],
                'addressLine1' => $input_data['address_line1'],
                'addressLine2' => $input_data['address_line2'],
                'city' => $input_data['city'],
                'stateOrRegion' => $input_data['state_code'],
                'postalCode' => $input_data['zip_code'],
                'countryCode' => $input_data['country_code'],
                'phone' => $input_data['phone']
                
            ],
            'shippingSpeedCategory' => $input_data['speed'],
            'items' => [
                [
                    'quantity' => $input_data['quantity'],
                    'sellerFulfillmentOrderItemId' => $input_data['order_no'],
                    'sellerSku' => $input_data['amazon_sku'],
                    'displayableComment' => 'Do not leave the package at the door step'
                ],
            ],
        ];
        // return $requestData;
        
        // dd($requestData);
        
        // Convert the request data to JSON
        $jsonData = json_encode($requestData);
        
        
        
        // Make the POST request to the SP API endpoint
        $response =  Http::withHeaders([
            'x-amz-access-token' => $access_token,
            'Content-Type' => 'application/json',
            'User-Agent' => $userAgent,
        ])->post($api_endpoint, $jsonData);
        
        $responseData = $response->json();
        $filePath=storage_path('amazonspapi/mcf/create_order.json');
        file_put_contents($filePath, json_encode($responseData, JSON_PRETTY_PRINT));
        $arr=[];
        if ($response->status() === 200) {
            $data=[
                'error' => false,
                'data' => $arr,
                'message' => 'successfully working'
            ]; 
        } else {
            $error=$responseData['errors'][0]['message'];
            $data=[
                'error' => true,
                'data' => $arr,
                'message' => $error
            ];
        }
        
        return $data;
        
    } 
    
    public static function getFullFillmentOrder($access_token,$marketplace_id,$input_data){
        
        $api_endpoint=self::$url.$input_data['order_no'];
        $userAgent = \Request::server('HTTP_USER_AGENT');
        
        $response =  Http::withHeaders([
            'x-amz-access-token' => $access_token,
            'Content-Type' => 'application/json',
            'User-Agent' => $userAgent,
        ])->get($api_endpoint);
        
        $responseData = $response->json();
        
        // dd($responseData);
        $arr=[];
        $orderstatus=$shipstatus=$carrier=$tracking_number=$ship_date='';
        if ($response->status() === 200) {
            $fullFillOrderDetail=$responseData['payload']['fulfillmentOrder'];
            $orderstatus=$fullFillOrderDetail['fulfillmentOrderStatus'];
            $ship_date=date('d-m-Y',strtotime($fullFillOrderDetail['displayableOrderDate']));
            
            // return $ship_date;
            
            if(isset($responseData['payload']['fulfillmentShipments'][0])){
                $fulfillmentShipments=$responseData['payload']['fulfillmentShipments'][0];
                $shipstatus=$fulfillmentShipments['fulfillmentShipmentStatus'];
            }
            
            
            
            if(isset($fulfillmentShipments['fulfillmentShipmentPackage'][0])){
                $shipmentPackage=$fulfillmentShipments['fulfillmentShipmentPackage'][0];
                $carrier=$shipmentPackage['carrierCode'];
                $tracking_number=$shipmentPackage['trackingNumber'];
            }
            
            $arr=[
                'amazon_order_status' => $orderstatus,
                'shipment_status' => $shipstatus,
                'tracking_number' => $tracking_number,
                'carrier' =>$carrier,
                'ship_date' => $ship_date
            ];
            
            
            $data=[
                'error' => false,
                'data' => $arr,
                'message' => 'successfully working'
            ];
        }else{
            $error=$responseData['errors'][0]['message'];
            $data=[
                'error' => true,
                'data' => $arr,
                'message' => $error
            ];
            
        }
        
        return $data;
        
        
        
    }
    
    public static function cancelFullFillment($access_token,$marketplace_id,$input_data){
        
        $api_endpoint=self::$url.$input_data['order_no'].'/cancel';
        $userAgent = \Request::server('HTTP_USER_AGENT');
        
        
        
        $response =  Http::withHeaders([
            'x-amz-access-token' => $access_token,
            'Content-Type' => 'application/json',
            'User-Agent' => $userAgent,
        ])->put($api_endpoint);
        // dd($response->status());
        $responseData = $response->json();
        
        $arr=[];
        if ($response->status() === 200) {
            // return 'new2';
            $data=[
                'error' => false,
                'data' => $arr,
                'message' => 'successfully working'
            ];
        }else{
            // return 'new';
            $error=$responseData['errors'][0]['message'];
            $data=[
                'error' => true,
                'data' => $arr,
                'message' => $error
            ];
             return $data;
        }
        
        return $data;
    }
    
    public static function submitFeed($access_token,$marketplace_ids,$input_data){
        echo $access_token;
        $api_endpoint=self::$feedurl.'/documents';
        $userAgent = \Request::server('HTTP_USER_AGENT');
        
        $feedType = 'POST_ORDER_FULFILLMENT_DATA';
        $marketplaceIds = $marketplace_ids; // Add more as needed
        
        // Convert the request data to JSON
        $requestData = [
            'feedType' => $feedType,
            'marketplaceIds' => $marketplaceIds,
            'contentType' => 'application/json',
        ];
        
        $jsonData = json_encode($requestData);

        // Make the POST request to the SP API endpoint
        $response = Http::withHeaders([
            'x-amz-access-token' => $access_token,
            'Content-Type' => 'application/json',
            'User-Agent' => $userAgent,
        ])->post($api_endpoint, $requestData);
        
        
        if($response->successful()){
            $responseData = $response->json();
            
            dd($responseData);
            $feedDocumentId=$responseData['feedDocumentId'];
            $url_content=$responseData['url'];
            
            $jsonData = [
                "OrderFulfillment" => [
                    "AmazonOrderID" => $input_data['reference_no'],
                    "FulfillmentDate" => $input_data['shipment_date'].'T05:00:00.000-00:00',
                    "FulfillmentData" => [
                        "CarrierName" => $input_data['carrier'],
                        "ShippingMethod" => $input_data['service'],
                        "ShipperTrackingNumber" => $input_data['tracking_number']
                    ]
                ]
            ];
            
            $feed_contents=json_encode($jsonData);
            
            $response = Http::withHeaders([
                'x-amz-access-token' => $access_token,
                'Content-Type' => 'application/json',
                'User-Agent' => $userAgent,
            ])->put($url_content, $feed_contents);
            
            if($response->successful()){
                $create_feed_url = self::$feedurl.'/feeds';
                $create_feed_data = [
                    'feedDocumentId' => $feedDocumentId,
                    'marketplaceIds' => $marketplaceIds,
                    'feedType' => $feedType,
                ];
                $response = Http::withHeaders([
                    'x-amz-access-token' => $access_token,
                    'Content-Type' => 'application/json',
                    'User-Agent' => $userAgent,
                ])->post($create_feed_url, $create_feed_data);
                
                
                $feed_id = $response->json()['feedId'];
                
                echo $feed_id;
                
                
            }else{
                $responseData = $response->json();
                dd($responseData);
                echo " Error 2 : ". $responseData['errors'][0]['message'];
            }
            
            
            
            
            
            
            
        }else{
            $responseData = $response->json();
            echo " Error 1 : ". $responseData['errors'][0]['message'];
        
        }
            
        
        
       

        
    } 
}
?>