<?php 
namespace App\Helpers;

use Illuminate\Support\Facades\DB;
use Http;
use Carbon;

class OrderHelper{

    public static function getAccessToken($clientId,$clientSecret,$refreshToken,$jsonFile)
    {
        $response = Http::asForm()->post('https://api.amazon.com/auth/o2/token', [
            'client_id' =>$clientId,
            'client_secret' => $clientSecret,
            'grant_type' => 'refresh_token',
            'refresh_token' => $refreshToken,
        ]);

        if ($response->successful()) {
            $tokenData = [
                'access_token' => $response['access_token'],
            ];
            file_put_contents($jsonFile, json_encode($tokenData, JSON_PRETTY_PRINT));

            return response()->json([
                'status' => true,
                'msg' => 'success',
            ]);
        } else {
            return response()->json([
                'status' => false,
                'data' => $response->json(),
            ]);
        }
    }

    public static function requestOrdReport($jsonFile)
    {
        $tokenFilePath = $jsonFile;
        $tokenData = json_decode(file_get_contents($tokenFilePath), true);
        $access_token = $tokenData['access_token'];
        $userAgent = \Request::server('HTTP_USER_AGENT');
       
       $now = now();
       
       $start = now()->subDays(30);
       
        $response = Http::withHeaders([
            'x-amz-access-token' => $access_token,
            'Content-Type' => 'application/json',
            'User-Agent' => $userAgent,
        ])->post('https://sellingpartnerapi-na.amazon.com/reports/2021-06-30/reports', [
            'reportType' => 'GET_FLAT_FILE_ALL_ORDERS_DATA_BY_LAST_UPDATE_GENERAL',
            'marketplaceIds' => ['ATVPDKIKX0DER','A2EUQ1WTGCTBG2'], 
            'dataStartTime' => $start,
            'dataEndTime' => $now,
        ]);
        
        if ($response->successful()) {
            
            $tokenData['reportId'] = $response['reportId'];
            file_put_contents($tokenFilePath, json_encode($tokenData, JSON_PRETTY_PRINT));

            return response()->json([
                'status' => true,
                'data' => $response->json(),
            ]);
        } else {
            return response()->json([
                'status' => false,
                'data' => $response->json(),
            ]);
        }
    }

    public static function checkOrdReportStatus($jsonFile)
    {
        $jsonFilePath = $jsonFile;
        $getData = json_decode(file_get_contents($jsonFilePath), true);

        $access_token = $getData['access_token'];
        $reportId = $getData['reportId'];
        $userAgent = \Request::server('HTTP_USER_AGENT');

        $response = Http::withHeaders([
            'x-amz-access-token' => $access_token,
            'Content-Type' => 'application/json',
            'User-Agent' => $userAgent,
        ])->get("https://sellingpartnerapi-na.amazon.com/reports/2021-06-30/reports/{$reportId}");

        if ($response->successful()) {
            $res = $response->json();
            if ($res['processingStatus'] == 'DONE') {
                $getData['documentId'] = $res['reportDocumentId'];
                file_put_contents($jsonFilePath, json_encode($getData, JSON_PRETTY_PRINT));
                return response()->json([
                    'status' => true,
                    'data' => $res,
                ]);
            } else {
                return response()->json([
                    'status' => true,
                    'data' => $res['processingStatus'],
                ]);
            }
        } else {
            return response()->json([
                'status' => false,
                'data' => $response->json(),
            ]);
        }
    }

    public static function downloadOrdReport($jsonFile,$resFile,$url)
    {
        $jsonFilePath = $jsonFile;
        $getData = json_decode(file_get_contents($jsonFilePath), true);

        $access_token = $getData['access_token'];
        $reportId = $getData['reportId'];
        $documentId = $getData['documentId'];
        $userAgent = \Request::server('HTTP_USER_AGENT');

        $response = Http::withHeaders([
            'x-amz-access-token' => $access_token,
            'Content-Type' => 'application/json',
            'User-Agent' => $userAgent,
        ])->get("https://sellingpartnerapi-na.amazon.com/reports/2021-06-30/documents/{$documentId}");

        if ($response->successful()) {

            $reportDownloadUrl = $response['url'];
            
            if(isset($response['compressionAlgorithm'])){
                $encryptedReport = file_get_contents($reportDownloadUrl);
                $decryptJson = gzdecode($encryptedReport);
                file_put_contents($resFile, $decryptJson);
            }else{
                $encryptedReport = file_get_contents($reportDownloadUrl);
                file_put_contents($resFile,$encryptedReport);  
            }
            
            $filepath = file_get_contents($resFile);
            
            $response = Http::withHeaders([
                    'Authorization' => 'Bearer n11hnvR1nd]RCxq-Ld5F^=#b!1{+w8q$Eod',
                ])->post($url, [
                    'orderData' => $filepath,
                ]);
            
            if($response->successful()){
                return response()->json([
                        'success' => true,
                        'message' => 'Report downloaded and decrypted successfully',
                    ]);         
            }
            else{
                return response()->json([
                        'success' => false,
                        'message' => $response
                    ]);       
            }
           
        } else {
            return response()->json([
                'status' => false,
                'data' => $response->json(),
            ]);
        }
    }
}
?>